require 'runit/testcase.rb'
require 'runit/cui/testrunner.rb'
require 'find'

require 'makehtml'

=begin
Unit test of RD2HTML class
=end

class TestRD2HTML < RUNIT::TestCase
  def setup
    @cur_dir = Dir.pwd
    @rd2html = RD2HTML.new('doc_en')
  end

  def test_out_file
    assert_equal('testcase', @rd2html.out_file('testcase.rd'))
    assert_equal('cui/testrunner', @rd2html.out_file('cui/testrunner.rd'))
  end

  def test_cmd_rd2html
    assert_equal('rd2 --output-index -otestcase testcase.rd',
                 @rd2html.cmd_rd2html('testcase.rd'))
		 
    assert_equal('rd2 --output-index -ocui/testrunner cui/testrunner.rd',
                 @rd2html.cmd_rd2html('cui/testrunner.rd'))
  end

  def test_create
    @rd2html.create('testcase.rd')
    assert(File.exist?('testcase.html'))
    assert(File.file?('testcase.html'))
    assert(File.exist?('testcase.rmi'))
    assert(File.file?('testcase.rmi'))
 
    @rd2html.create('cui/testrunner.rd')
    assert(File.exist?('cui/testrunner.html'))
    assert(File.file?('cui/testrunner.html'))
    assert(File.exist?('cui/testrunner.rmi'))
    assert(File.file?('cui/testrunner.rmi'))
  end

  def test_cmd_rmi2html
    @rd2html.create('testcase.rd')
    assert_equal('rmi2html.rb testcase.rmi > index.html', @rd2html.cmd_rmi2html)
    @rd2html.create('cui/testrunner.rd')
    assert_equal('rmi2html.rb testcase.rmi cui/testrunner.rmi > index.html',
                 @rd2html.cmd_rmi2html)
  end

  def test_create_index_file
    @rd2html.create('testcase.rd')
    @rd2html.create('cui/testrunner.rd')
    @rd2html.create_index_file
    assert(File.file?('index.html'))
  end

  def teardown
    Dir.chdir(@cur_dir)
    Find.find('doc_en') do |f|
      if /\.html/ =~ f || /\.rmi/ =~ f
	File.unlink(f)
      end
    end
  end
end

RUNIT::CUI::TestRunner.run(TestRD2HTML.suite)
