require 'runit/testcase'
require 'runit/cui/testrunner'
require 'runit/topublic'

class ClassToPublic
  include RUNIT::ToPublic
end

class Dummy
  def private_foo
  end
  private :private_foo

  def protected_bar
  end
  protected :protected_bar

  def Dummy.to_private
    private :private_foo
  end

  def Dummy.to_protected
    protected :protected_bar
  end
end

class TestToPublic < RUNIT::TestCase
  def setup
    @to_pub = ClassToPublic.new
  end

  def test_to_public
    methods = Dummy.private_instance_methods
    assert(methods.include?('private_foo'))
    k = @to_pub.to_public(Dummy)

    assert_equal('Dummy', k.superclass.name)
    methods = k.public_instance_methods
    assert(methods.include?('private_foo'))
    assert(methods.include?('protected_bar'))

    methods = Dummy.private_instance_methods
    assert(methods.include?('private_foo'))
    methods = Dummy.protected_instance_methods
    assert(methods.include?('protected_bar'))
  end

  def teardown
    Dummy.to_private
    Dummy.to_protected
  end
end

if $0 == __FILE__
  RUNIT::CUI::TestRunner.run(TestToPublic.suite)
end
